/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "__generics.h"
#include "vtype.h"

#ifndef LIBCDSB_DICT_H
#define LIBCDSB_DICT_H

/*#####################################################################################################################*/

typedef int (*dict_access_callback)(const void* key, vtype key_type, void* value, vtype value_type, void* data);

extern void dict_init(vtype_dict* x) Nonnull__(1);

#define dict_pop(x, key, data, callback) _LIBCDSB_Generic (libcdsb_dict, get_by, key)(x, key, data, callback, 1)
#define dict_get(x, key, data, callback) _LIBCDSB_Generic (libcdsb_dict, get_by, key)(x, key, data, callback, 0)
#define dict_update(x, key, value)       _LIBCDSB_Generic2(libcdsb_dict, update, key, value)(x, key, value)
#define dict_remove(x, key)              dict_pop(x, key, 0, 0)

#define in_dict(x, key)                  (dict_get(&x, key, 0, 0) == 0)

/*#####################################################################################################################*/

extern int libcdsb_dict_get_by_pointer(vtype_dict* x, const         void* key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_cstring(vtype_dict* x, const         char* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_string (vtype_dict* x, const vtype_string* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_array  (vtype_dict* x, const  vtype_array* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_list   (vtype_dict* x, const   vtype_list* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_map    (vtype_dict* x, const    vtype_map* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_vset   (vtype_dict* x, const    vtype_set* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_dict   (vtype_dict* x, const   vtype_dict* key, void* data, dict_access_callback, bool cut) Nonnull__(1,2);
extern int libcdsb_dict_get_by_boolean(vtype_dict* x,          vtype_bool key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_int8   (vtype_dict* x,          vtype_int8 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_int16  (vtype_dict* x,         vtype_int16 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_int32  (vtype_dict* x,         vtype_int32 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_int64  (vtype_dict* x,         vtype_int64 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_uint8  (vtype_dict* x,         vtype_uint8 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_uint16 (vtype_dict* x,        vtype_uint16 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_uint32 (vtype_dict* x,        vtype_uint32 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_uint64 (vtype_dict* x,        vtype_uint64 key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_float  (vtype_dict* x,         vtype_float key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_double (vtype_dict* x,        vtype_double key, void* data, dict_access_callback, bool cut) Nonnull__(1);
extern int libcdsb_dict_get_by_ldouble(vtype_dict* x,       vtype_ldouble key, void* data, dict_access_callback, bool cut) Nonnull__(1);

extern bool libcdsb_dict_update_pointer_pointer(vtype_dict* x, const void* key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_cstring(vtype_dict* x, const void* key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_string (vtype_dict* x, const void* key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_array  (vtype_dict* x, const void* key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_list   (vtype_dict* x, const void* key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_map    (vtype_dict* x, const void* key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_vset   (vtype_dict* x, const void* key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_dict   (vtype_dict* x, const void* key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_pointer_boolean(vtype_dict* x, const void* key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_int8   (vtype_dict* x, const void* key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_int16  (vtype_dict* x, const void* key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_int32  (vtype_dict* x, const void* key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_int64  (vtype_dict* x, const void* key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_uint8  (vtype_dict* x, const void* key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_uint16 (vtype_dict* x, const void* key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_uint32 (vtype_dict* x, const void* key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_uint64 (vtype_dict* x, const void* key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_float  (vtype_dict* x, const void* key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_double (vtype_dict* x, const void* key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_pointer_ldouble(vtype_dict* x, const void* key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_cstring_pointer(vtype_dict* x, const char* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_cstring(vtype_dict* x, const char* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_string (vtype_dict* x, const char* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_array  (vtype_dict* x, const char* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_list   (vtype_dict* x, const char* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_map    (vtype_dict* x, const char* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_vset   (vtype_dict* x, const char* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_dict   (vtype_dict* x, const char* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_cstring_boolean(vtype_dict* x, const char* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_int8   (vtype_dict* x, const char* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_int16  (vtype_dict* x, const char* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_int32  (vtype_dict* x, const char* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_int64  (vtype_dict* x, const char* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_uint8  (vtype_dict* x, const char* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_uint16 (vtype_dict* x, const char* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_uint32 (vtype_dict* x, const char* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_uint64 (vtype_dict* x, const char* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_float  (vtype_dict* x, const char* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_double (vtype_dict* x, const char* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_cstring_ldouble(vtype_dict* x, const char* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_string_pointer(vtype_dict* x, const vtype_string* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_cstring(vtype_dict* x, const vtype_string* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_string (vtype_dict* x, const vtype_string* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_array  (vtype_dict* x, const vtype_string* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_list   (vtype_dict* x, const vtype_string* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_map    (vtype_dict* x, const vtype_string* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_vset   (vtype_dict* x, const vtype_string* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_dict   (vtype_dict* x, const vtype_string* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_string_boolean(vtype_dict* x, const vtype_string* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_int8   (vtype_dict* x, const vtype_string* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_int16  (vtype_dict* x, const vtype_string* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_int32  (vtype_dict* x, const vtype_string* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_int64  (vtype_dict* x, const vtype_string* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_uint8  (vtype_dict* x, const vtype_string* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_uint16 (vtype_dict* x, const vtype_string* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_uint32 (vtype_dict* x, const vtype_string* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_uint64 (vtype_dict* x, const vtype_string* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_float  (vtype_dict* x, const vtype_string* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_double (vtype_dict* x, const vtype_string* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_string_ldouble(vtype_dict* x, const vtype_string* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_array_pointer(vtype_dict* x, const vtype_array* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_cstring(vtype_dict* x, const vtype_array* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_string (vtype_dict* x, const vtype_array* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_array  (vtype_dict* x, const vtype_array* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_list   (vtype_dict* x, const vtype_array* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_map    (vtype_dict* x, const vtype_array* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_vset   (vtype_dict* x, const vtype_array* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_dict   (vtype_dict* x, const vtype_array* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_array_boolean(vtype_dict* x, const vtype_array* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_int8   (vtype_dict* x, const vtype_array* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_int16  (vtype_dict* x, const vtype_array* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_int32  (vtype_dict* x, const vtype_array* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_int64  (vtype_dict* x, const vtype_array* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_uint8  (vtype_dict* x, const vtype_array* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_uint16 (vtype_dict* x, const vtype_array* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_uint32 (vtype_dict* x, const vtype_array* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_uint64 (vtype_dict* x, const vtype_array* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_float  (vtype_dict* x, const vtype_array* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_double (vtype_dict* x, const vtype_array* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_array_ldouble(vtype_dict* x, const vtype_array* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_list_pointer(vtype_dict* x, const vtype_list* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_cstring(vtype_dict* x, const vtype_list* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_string (vtype_dict* x, const vtype_list* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_array  (vtype_dict* x, const vtype_list* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_list   (vtype_dict* x, const vtype_list* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_map    (vtype_dict* x, const vtype_list* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_vset   (vtype_dict* x, const vtype_list* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_dict   (vtype_dict* x, const vtype_list* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_list_boolean(vtype_dict* x, const vtype_list* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_int8   (vtype_dict* x, const vtype_list* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_int16  (vtype_dict* x, const vtype_list* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_int32  (vtype_dict* x, const vtype_list* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_int64  (vtype_dict* x, const vtype_list* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_uint8  (vtype_dict* x, const vtype_list* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_uint16 (vtype_dict* x, const vtype_list* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_uint32 (vtype_dict* x, const vtype_list* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_uint64 (vtype_dict* x, const vtype_list* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_float  (vtype_dict* x, const vtype_list* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_double (vtype_dict* x, const vtype_list* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_list_ldouble(vtype_dict* x, const vtype_list* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_map_pointer(vtype_dict* x, const vtype_map* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_cstring(vtype_dict* x, const vtype_map* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_string (vtype_dict* x, const vtype_map* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_array  (vtype_dict* x, const vtype_map* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_list   (vtype_dict* x, const vtype_map* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_map    (vtype_dict* x, const vtype_map* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_vset   (vtype_dict* x, const vtype_map* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_dict   (vtype_dict* x, const vtype_map* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_map_boolean(vtype_dict* x, const vtype_map* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_int8   (vtype_dict* x, const vtype_map* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_int16  (vtype_dict* x, const vtype_map* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_int32  (vtype_dict* x, const vtype_map* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_int64  (vtype_dict* x, const vtype_map* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_uint8  (vtype_dict* x, const vtype_map* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_uint16 (vtype_dict* x, const vtype_map* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_uint32 (vtype_dict* x, const vtype_map* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_uint64 (vtype_dict* x, const vtype_map* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_float  (vtype_dict* x, const vtype_map* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_double (vtype_dict* x, const vtype_map* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_map_ldouble(vtype_dict* x, const vtype_map* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_vset_pointer(vtype_dict* x, const vtype_set* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_cstring(vtype_dict* x, const vtype_set* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_string (vtype_dict* x, const vtype_set* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_array  (vtype_dict* x, const vtype_set* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_list   (vtype_dict* x, const vtype_set* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_map    (vtype_dict* x, const vtype_set* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_vset   (vtype_dict* x, const vtype_set* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_dict   (vtype_dict* x, const vtype_set* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_vset_boolean(vtype_dict* x, const vtype_set* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_int8   (vtype_dict* x, const vtype_set* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_int16  (vtype_dict* x, const vtype_set* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_int32  (vtype_dict* x, const vtype_set* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_int64  (vtype_dict* x, const vtype_set* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_uint8  (vtype_dict* x, const vtype_set* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_uint16 (vtype_dict* x, const vtype_set* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_uint32 (vtype_dict* x, const vtype_set* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_uint64 (vtype_dict* x, const vtype_set* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_float  (vtype_dict* x, const vtype_set* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_double (vtype_dict* x, const vtype_set* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_vset_ldouble(vtype_dict* x, const vtype_set* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_dict_pointer(vtype_dict* x, const vtype_dict* key, const         void* value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_cstring(vtype_dict* x, const vtype_dict* key, const         char* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_string (vtype_dict* x, const vtype_dict* key, const vtype_string* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_array  (vtype_dict* x, const vtype_dict* key, const  vtype_array* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_list   (vtype_dict* x, const vtype_dict* key, const   vtype_list* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_map    (vtype_dict* x, const vtype_dict* key, const    vtype_map* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_vset   (vtype_dict* x, const vtype_dict* key, const    vtype_set* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_dict   (vtype_dict* x, const vtype_dict* key, const   vtype_dict* value) Nonnull__(1,2,3);
extern bool libcdsb_dict_update_dict_boolean(vtype_dict* x, const vtype_dict* key,          vtype_bool value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_int8   (vtype_dict* x, const vtype_dict* key,          vtype_int8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_int16  (vtype_dict* x, const vtype_dict* key,         vtype_int16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_int32  (vtype_dict* x, const vtype_dict* key,         vtype_int32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_int64  (vtype_dict* x, const vtype_dict* key,         vtype_int64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_uint8  (vtype_dict* x, const vtype_dict* key,         vtype_uint8 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_uint16 (vtype_dict* x, const vtype_dict* key,        vtype_uint16 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_uint32 (vtype_dict* x, const vtype_dict* key,        vtype_uint32 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_uint64 (vtype_dict* x, const vtype_dict* key,        vtype_uint64 value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_float  (vtype_dict* x, const vtype_dict* key,         vtype_float value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_double (vtype_dict* x, const vtype_dict* key,        vtype_double value) Nonnull__(1,2);
extern bool libcdsb_dict_update_dict_ldouble(vtype_dict* x, const vtype_dict* key,       vtype_ldouble value) Nonnull__(1,2);

extern bool libcdsb_dict_update_boolean_pointer(vtype_dict* x, vtype_bool key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_cstring(vtype_dict* x, vtype_bool key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_string (vtype_dict* x, vtype_bool key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_array  (vtype_dict* x, vtype_bool key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_list   (vtype_dict* x, vtype_bool key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_map    (vtype_dict* x, vtype_bool key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_vset   (vtype_dict* x, vtype_bool key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_dict   (vtype_dict* x, vtype_bool key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_boolean_boolean(vtype_dict* x, vtype_bool key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_int8   (vtype_dict* x, vtype_bool key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_int16  (vtype_dict* x, vtype_bool key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_int32  (vtype_dict* x, vtype_bool key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_int64  (vtype_dict* x, vtype_bool key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_uint8  (vtype_dict* x, vtype_bool key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_uint16 (vtype_dict* x, vtype_bool key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_uint32 (vtype_dict* x, vtype_bool key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_uint64 (vtype_dict* x, vtype_bool key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_float  (vtype_dict* x, vtype_bool key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_double (vtype_dict* x, vtype_bool key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_boolean_ldouble(vtype_dict* x, vtype_bool key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_uint8_pointer(vtype_dict* x, vtype_uint8 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_cstring(vtype_dict* x, vtype_uint8 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_string (vtype_dict* x, vtype_uint8 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_array  (vtype_dict* x, vtype_uint8 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_list   (vtype_dict* x, vtype_uint8 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_map    (vtype_dict* x, vtype_uint8 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_vset   (vtype_dict* x, vtype_uint8 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_dict   (vtype_dict* x, vtype_uint8 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint8_boolean(vtype_dict* x, vtype_uint8 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_int8   (vtype_dict* x, vtype_uint8 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_int16  (vtype_dict* x, vtype_uint8 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_int32  (vtype_dict* x, vtype_uint8 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_int64  (vtype_dict* x, vtype_uint8 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_uint8  (vtype_dict* x, vtype_uint8 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_uint16 (vtype_dict* x, vtype_uint8 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_uint32 (vtype_dict* x, vtype_uint8 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_uint64 (vtype_dict* x, vtype_uint8 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_float  (vtype_dict* x, vtype_uint8 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_double (vtype_dict* x, vtype_uint8 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_uint8_ldouble(vtype_dict* x, vtype_uint8 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_uint16_pointer(vtype_dict* x, vtype_uint16 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_cstring(vtype_dict* x, vtype_uint16 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_string (vtype_dict* x, vtype_uint16 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_array  (vtype_dict* x, vtype_uint16 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_list   (vtype_dict* x, vtype_uint16 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_map    (vtype_dict* x, vtype_uint16 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_vset   (vtype_dict* x, vtype_uint16 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_dict   (vtype_dict* x, vtype_uint16 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint16_boolean(vtype_dict* x, vtype_uint16 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_int8   (vtype_dict* x, vtype_uint16 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_int16  (vtype_dict* x, vtype_uint16 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_int32  (vtype_dict* x, vtype_uint16 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_int64  (vtype_dict* x, vtype_uint16 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_uint8  (vtype_dict* x, vtype_uint16 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_uint16 (vtype_dict* x, vtype_uint16 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_uint32 (vtype_dict* x, vtype_uint16 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_uint64 (vtype_dict* x, vtype_uint16 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_float  (vtype_dict* x, vtype_uint16 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_double (vtype_dict* x, vtype_uint16 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_uint16_ldouble(vtype_dict* x, vtype_uint16 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_uint32_pointer(vtype_dict* x, vtype_uint32 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_cstring(vtype_dict* x, vtype_uint32 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_string (vtype_dict* x, vtype_uint32 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_array  (vtype_dict* x, vtype_uint32 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_list   (vtype_dict* x, vtype_uint32 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_map    (vtype_dict* x, vtype_uint32 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_vset   (vtype_dict* x, vtype_uint32 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_dict   (vtype_dict* x, vtype_uint32 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint32_boolean(vtype_dict* x, vtype_uint32 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_int8   (vtype_dict* x, vtype_uint32 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_int16  (vtype_dict* x, vtype_uint32 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_int32  (vtype_dict* x, vtype_uint32 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_int64  (vtype_dict* x, vtype_uint32 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_uint8  (vtype_dict* x, vtype_uint32 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_uint16 (vtype_dict* x, vtype_uint32 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_uint32 (vtype_dict* x, vtype_uint32 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_uint64 (vtype_dict* x, vtype_uint32 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_float  (vtype_dict* x, vtype_uint32 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_double (vtype_dict* x, vtype_uint32 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_uint32_ldouble(vtype_dict* x, vtype_uint32 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_uint64_pointer(vtype_dict* x, vtype_uint64 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_cstring(vtype_dict* x, vtype_uint64 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_string (vtype_dict* x, vtype_uint64 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_array  (vtype_dict* x, vtype_uint64 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_list   (vtype_dict* x, vtype_uint64 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_map    (vtype_dict* x, vtype_uint64 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_vset   (vtype_dict* x, vtype_uint64 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_dict   (vtype_dict* x, vtype_uint64 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_uint64_boolean(vtype_dict* x, vtype_uint64 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_int8   (vtype_dict* x, vtype_uint64 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_int16  (vtype_dict* x, vtype_uint64 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_int32  (vtype_dict* x, vtype_uint64 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_int64  (vtype_dict* x, vtype_uint64 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_uint8  (vtype_dict* x, vtype_uint64 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_uint16 (vtype_dict* x, vtype_uint64 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_uint32 (vtype_dict* x, vtype_uint64 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_uint64 (vtype_dict* x, vtype_uint64 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_float  (vtype_dict* x, vtype_uint64 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_double (vtype_dict* x, vtype_uint64 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_uint64_ldouble(vtype_dict* x, vtype_uint64 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_int8_pointer(vtype_dict* x, vtype_int8 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_cstring(vtype_dict* x, vtype_int8 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_string (vtype_dict* x, vtype_int8 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_array  (vtype_dict* x, vtype_int8 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_list   (vtype_dict* x, vtype_int8 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_map    (vtype_dict* x, vtype_int8 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_vset   (vtype_dict* x, vtype_int8 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_dict   (vtype_dict* x, vtype_int8 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int8_boolean(vtype_dict* x, vtype_int8 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_int8   (vtype_dict* x, vtype_int8 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_int16  (vtype_dict* x, vtype_int8 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_int32  (vtype_dict* x, vtype_int8 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_int64  (vtype_dict* x, vtype_int8 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_uint8  (vtype_dict* x, vtype_int8 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_uint16 (vtype_dict* x, vtype_int8 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_uint32 (vtype_dict* x, vtype_int8 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_uint64 (vtype_dict* x, vtype_int8 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_float  (vtype_dict* x, vtype_int8 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_double (vtype_dict* x, vtype_int8 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_int8_ldouble(vtype_dict* x, vtype_int8 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_int16_pointer(vtype_dict* x, vtype_int16 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_cstring(vtype_dict* x, vtype_int16 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_string (vtype_dict* x, vtype_int16 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_array  (vtype_dict* x, vtype_int16 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_list   (vtype_dict* x, vtype_int16 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_map    (vtype_dict* x, vtype_int16 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_vset   (vtype_dict* x, vtype_int16 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_dict   (vtype_dict* x, vtype_int16 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int16_boolean(vtype_dict* x, vtype_int16 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_int8   (vtype_dict* x, vtype_int16 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_int16  (vtype_dict* x, vtype_int16 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_int32  (vtype_dict* x, vtype_int16 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_int64  (vtype_dict* x, vtype_int16 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_uint8  (vtype_dict* x, vtype_int16 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_uint16 (vtype_dict* x, vtype_int16 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_uint32 (vtype_dict* x, vtype_int16 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_uint64 (vtype_dict* x, vtype_int16 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_float  (vtype_dict* x, vtype_int16 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_double (vtype_dict* x, vtype_int16 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_int16_ldouble(vtype_dict* x, vtype_int16 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_int32_pointer(vtype_dict* x, vtype_int32 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_cstring(vtype_dict* x, vtype_int32 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_string (vtype_dict* x, vtype_int32 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_array  (vtype_dict* x, vtype_int32 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_list   (vtype_dict* x, vtype_int32 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_map    (vtype_dict* x, vtype_int32 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_vset   (vtype_dict* x, vtype_int32 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_dict   (vtype_dict* x, vtype_int32 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int32_boolean(vtype_dict* x, vtype_int32 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_int8   (vtype_dict* x, vtype_int32 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_int16  (vtype_dict* x, vtype_int32 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_int32  (vtype_dict* x, vtype_int32 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_int64  (vtype_dict* x, vtype_int32 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_uint8  (vtype_dict* x, vtype_int32 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_uint16 (vtype_dict* x, vtype_int32 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_uint32 (vtype_dict* x, vtype_int32 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_uint64 (vtype_dict* x, vtype_int32 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_float  (vtype_dict* x, vtype_int32 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_double (vtype_dict* x, vtype_int32 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_int32_ldouble(vtype_dict* x, vtype_int32 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_int64_pointer(vtype_dict* x, vtype_int64 key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_cstring(vtype_dict* x, vtype_int64 key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_string (vtype_dict* x, vtype_int64 key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_array  (vtype_dict* x, vtype_int64 key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_list   (vtype_dict* x, vtype_int64 key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_map    (vtype_dict* x, vtype_int64 key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_vset   (vtype_dict* x, vtype_int64 key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_dict   (vtype_dict* x, vtype_int64 key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_int64_boolean(vtype_dict* x, vtype_int64 key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_int8   (vtype_dict* x, vtype_int64 key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_int16  (vtype_dict* x, vtype_int64 key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_int32  (vtype_dict* x, vtype_int64 key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_int64  (vtype_dict* x, vtype_int64 key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_uint8  (vtype_dict* x, vtype_int64 key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_uint16 (vtype_dict* x, vtype_int64 key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_uint32 (vtype_dict* x, vtype_int64 key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_uint64 (vtype_dict* x, vtype_int64 key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_float  (vtype_dict* x, vtype_int64 key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_double (vtype_dict* x, vtype_int64 key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_int64_ldouble(vtype_dict* x, vtype_int64 key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_float_pointer(vtype_dict* x, vtype_float key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_float_cstring(vtype_dict* x, vtype_float key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_string (vtype_dict* x, vtype_float key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_array  (vtype_dict* x, vtype_float key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_list   (vtype_dict* x, vtype_float key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_map    (vtype_dict* x, vtype_float key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_vset   (vtype_dict* x, vtype_float key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_dict   (vtype_dict* x, vtype_float key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_float_boolean(vtype_dict* x, vtype_float key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_float_int8   (vtype_dict* x, vtype_float key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_int16  (vtype_dict* x, vtype_float key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_int32  (vtype_dict* x, vtype_float key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_int64  (vtype_dict* x, vtype_float key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_uint8  (vtype_dict* x, vtype_float key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_uint16 (vtype_dict* x, vtype_float key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_uint32 (vtype_dict* x, vtype_float key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_uint64 (vtype_dict* x, vtype_float key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_float_float  (vtype_dict* x, vtype_float key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_float_double (vtype_dict* x, vtype_float key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_float_ldouble(vtype_dict* x, vtype_float key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_double_pointer(vtype_dict* x, vtype_double key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_double_cstring(vtype_dict* x, vtype_double key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_string (vtype_dict* x, vtype_double key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_array  (vtype_dict* x, vtype_double key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_list   (vtype_dict* x, vtype_double key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_map    (vtype_dict* x, vtype_double key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_vset   (vtype_dict* x, vtype_double key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_dict   (vtype_dict* x, vtype_double key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_double_boolean(vtype_dict* x, vtype_double key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_double_int8   (vtype_dict* x, vtype_double key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_int16  (vtype_dict* x, vtype_double key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_int32  (vtype_dict* x, vtype_double key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_int64  (vtype_dict* x, vtype_double key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_uint8  (vtype_dict* x, vtype_double key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_uint16 (vtype_dict* x, vtype_double key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_uint32 (vtype_dict* x, vtype_double key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_uint64 (vtype_dict* x, vtype_double key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_double_float  (vtype_dict* x, vtype_double key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_double_double (vtype_dict* x, vtype_double key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_double_ldouble(vtype_dict* x, vtype_double key,       vtype_ldouble value) Nonnull__(1);

extern bool libcdsb_dict_update_ldouble_pointer(vtype_dict* x, vtype_ldouble key, const         void* value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_cstring(vtype_dict* x, vtype_ldouble key, const         char* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_string (vtype_dict* x, vtype_ldouble key, const vtype_string* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_array  (vtype_dict* x, vtype_ldouble key, const  vtype_array* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_list   (vtype_dict* x, vtype_ldouble key, const   vtype_list* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_map    (vtype_dict* x, vtype_ldouble key, const    vtype_map* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_vset   (vtype_dict* x, vtype_ldouble key, const    vtype_set* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_dict   (vtype_dict* x, vtype_ldouble key, const   vtype_dict* value) Nonnull__(1,3);
extern bool libcdsb_dict_update_ldouble_boolean(vtype_dict* x, vtype_ldouble key,          vtype_bool value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_int8   (vtype_dict* x, vtype_ldouble key,          vtype_int8 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_int16  (vtype_dict* x, vtype_ldouble key,         vtype_int16 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_int32  (vtype_dict* x, vtype_ldouble key,         vtype_int32 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_int64  (vtype_dict* x, vtype_ldouble key,         vtype_int64 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_uint8  (vtype_dict* x, vtype_ldouble key,         vtype_uint8 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_uint16 (vtype_dict* x, vtype_ldouble key,        vtype_uint16 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_uint32 (vtype_dict* x, vtype_ldouble key,        vtype_uint32 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_uint64 (vtype_dict* x, vtype_ldouble key,        vtype_uint64 value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_float  (vtype_dict* x, vtype_ldouble key,         vtype_float value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_double (vtype_dict* x, vtype_ldouble key,        vtype_double value) Nonnull__(1);
extern bool libcdsb_dict_update_ldouble_ldouble(vtype_dict* x, vtype_ldouble key,       vtype_ldouble value) Nonnull__(1);

#endif /* LIBCDSB_DICT_H */
