/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

/*#####################################################################################################################*/

hash_t dict_hash(const dict_t* s) {
    dnode_t *min, *max;
    size_t           i;
    dnode_t         *c;
    hash_t        hash;

    if (!s->size)
        return 0;

    min = max = nullptr;
    i   = s->capacity;

    while (i--) {
        c = s->nodes[i];

        while (!is_null(c)) {
            if (is_null(min) || vnode_compare(&c->key, c->key_type, &min->key, min->key_type) < 0) min = c;
            if (is_null(max) || vnode_compare(&c->key, c->key_type, &max->key, max->key_type) > 0) max = c;

            c = c->prev;
        }
    }

    hash = vnode_hash(&min->key, min->key_type);

    if (s->size > 0)
        hash += vnode_hash(&max->key, max->key_type);

    return (hash ^ s->size) + VTYPE_DICT;
}


void dict_init(dict_t* x) {
    memset(x, 0, sizeof(*x));
}

/*#####################################################################################################################*/

void dict_free(dict_t* x) {

    while (x->capacity--) {
        while (!is_null(x->nodes[x->capacity])) {
            vnode_free(&x->nodes[x->capacity]->key,   x->nodes[x->capacity]->key_type);
            vnode_free(&x->nodes[x->capacity]->value, x->nodes[x->capacity]->value_type);

            x->nodes[x->capacity] = x->nodes[x->capacity]->prev;
        }
    }

    free(x->nodes);
    memset(x, 0, sizeof(*x));
}

/*#####################################################################################################################*/

size_t dict_size(const dict_t* x) {
    return x->size;
}

size_t dict_capacity(const dict_t* x) {
    return x->capacity;
}

/*#####################################################################################################################*/

int dict_compare(const dict_t* s0, const dict_t* s1) {
    dnode_t *c0, *c1;
    size_t         i;
    int          cmp;

    if (s0 == s1)
        return 0;

    if (s0->size != s1->size)
        return s0->size < s1->size ? -1 : 1;

    i = s0->capacity;

    while (i--) {
        c0 = s0->nodes[i];

        while (!is_null(c0)) {

            c1  = s1->nodes[vnode_hash(&c0->key, c0->key_type) % s1->capacity];
            cmp = -1;

            while (!is_null(c1)) {
                if ((cmp = vnode_compare(&c0->key, c0->key_type, &c1->key, c1->key_type) == 0)) {

                    cmp = vnode_compare(&c0->value, c0->value_type, &c1->value, c1->value_type);
                    break;
                }

                c1 = c1->prev;
            }

            if (cmp) return cmp;

            c0 = c0->prev;
        }
    }

    return 0;
}
