/* This software is licensed by the MIT License, see LICENSE file */
/*                                Copyright © 2022 Gregory Lirent */

#include "include.h"

int libcdsb_map_find(map_t* x, const void* k, vtype t, void* _, map_access_callback callback, bool cut) {
    mnode_t*  c;
    void   *key;
    int     cmp;

    c = x->root;

    while (!mnode_is_empty(c)) {
        key = vnode_peek(&c->key, x->type);
        cmp = vtype_compare(k, t, key, x->type);

        if (cmp == 0) {
            cmp = (callback) ? callback(key, x->type, vnode_peek(&c->value, c->type), c->type, _) : 0;

            if (cut) {
                c = mnode_delete(&x->root, c);
                vnode_free(&c->key,   x->type);
                vnode_free(&c->value, c->type);
                free(c);
            }

            return cmp;
        } else c = (cmp < 0) ? c->left : c->right;
    }

    return -1;
}


int libcdsb_map_foreach(map_t* x, void* dt, map_access_callback callback, bool flush) {
    stack_t  z;
    int      r;
    mnode_t* c;

    stack_init(&z);
    stack_push(&z, x->root);

    r = 0;

    if (mnode_is_empty(x->root))
        return 0;

    while ((c = stack_pop(&z))) {
        if ((r = callback(vnode_peek(&c->key, x->type), x->type, vnode_peek(&c->value, c->type), c->type, dt)))
            break;

        if (!mnode_is_empty(c->right)) stack_push(&z, c->right);
        if (!mnode_is_empty(c->left))  stack_push(&z, c->left);

        if (flush) {
            vnode_free(&c->key,   x->type);
            vnode_free(&c->value, c->type);
            free(c);
        }
    }

    if (flush) {
        while (c) {
            if (!mnode_is_empty(c->right)) stack_push(&z, c->right);
            if (!mnode_is_empty(c->left))  stack_push(&z, c->left);

            vnode_free(&c->key,   x->type);
            vnode_free(&c->value, c->type);
            free(c);

            c = stack_pop(&z);
        }

        memset(x, 0, sizeof(*x));
    } else stack_flush(&z);

    return r;
}
